gettext = function(s) {
  return s;
};

// Inject application dependencies here
angular.module('app', [
  'ui.router',
  'ui.bootstrap',
  'ngAnimate',
  'templates',
  'gettext',
  'chart.js',
  'ngCookies',
  'leaflet-directive'])

// Configure the default route
.config(function($urlRouterProvider, $stateProvider, $sceProvider) {
  // TODO: add a reference to a 401 page here (for now I will point to "home")
  $urlRouterProvider.otherwise('/app/home');
})

.run(function($rootScope, $location) {

})

.config(function($stateProvider) {
  $stateProvider
    /* top navigation bar */
    .state('app.example', {
      url: "/example",
      views: {
        'content': {
          templateUrl: "src/example/example.html",
          controller: "exampleController"
        }
      }
    });
})

.controller('exampleController', function($scope, $state, $window) {

})

.factory('gaugeFactory', function($http, $q, $rootScope, $timeout) {
  var endpoint = 'data_cache/gauges.json';

  return {
  	get: function() {
  	  var deferred = $q.defer();
  	  $http.get(endpoint)
  	    .success(function(data) {
  	      deferred.resolve(data);
  	    })
  	    .error(function(msg, code) {
  	      deferred.reject(msg);
  	    });
  	  return deferred.promise;
  	}
  }
})

.config(function($stateProvider) {
  $stateProvider
    /* top navigation bar */
    .state('app.home', {
      url: "/home",
      views: {
        'content': {
          templateUrl: "src/home/home.html",
          controller: "homeController"
        }
      }
    });
})

.config(function(ChartJsProvider) {
  ChartJsProvider.setOptions({chartColors : [ '#337ab7', '#5bc0de', '#f0ad4e', '#d9534f' ]});
})

.constant('RIVER_STATES', {
  'NORMAL': 0x01,
  'ACTION': 0X02,
  'FLOOD':  0X03,
  'MAJOR':  0X04
})

.constant('DEFAULT_OVERRIDES', {
  fill: false,
  pointRadius: 0,
  pointHitRadius: 25,
  borderWidth: 10,
  pointHoverBorderWidth: 3
})

.constant('MINUTE_TICKS', 60000)

.controller('homeController', function($scope, $state, $window, $filter, $cookies, $interval, gaugeFactory, RIVER_STATES, MINUTE_TICKS, DEFAULT_OVERRIDES) {

  /* These objects are for the hydrographs */
  $scope.charts = [];
  $scope.series = ["River Level", "Action Stage", "Flood Stage", "Major Flood"];
  $scope.datasetOverride = [];
  $scope.datasetOverride[0] = {pointRadius:0, pointHitRadius: 10}
  $scope.datasetOverride[1] = DEFAULT_OVERRIDES;
  $scope.datasetOverride[2] = DEFAULT_OVERRIDES;
  $scope.datasetOverride[3] = DEFAULT_OVERRIDES;

  function init_status() {
    $scope.status = {};
    var my_cookie = $cookies.getObject('visibility');

    if (typeof(my_cookie) !== 'undefined') {
      $scope.status = my_cookie;
    }
  }

  init_status();

  $scope.on_visibility_change = function(newVal) {
    $cookies.putObject('visibility', newVal);
  }

  /* Compare a river height to the gauge stages */
  function get_river_state(height, stages) {
    var state = RIVER_STATES.NORMAL;
    var my_height = parseFloat(height);

    if (my_height >= parseFloat(stages.major)) {
      state = RIVER_STATES.MAJOR;
    } else if (my_height >= parseFloat(stages.flood)) {
      state = RIVER_STATES.FLOOD;
    } else if (my_height >= parseFloat(stages.action)) {
      state = RIVER_STATES.ACTION;
    }
    return state;
  }

  /* Add a current river level property */
  function add_current_river_info(data) {
    for (var id in data) {
      data[id].current_level = data[id].observed.datum[0].primary;
      data[id].current_time  = data[id].observed.datum[0].valid;
      data[id].current_state = get_river_state(data[id].current_level, data[id].sigstages);
    }
  }

  /* Conditionally add a crest property */
  function add_crest_info(data) {
    for (var id in data) {
      var crest_height = 0;
      var crest_i      = 0;
      
      for (var i = 0; i < data[id].forecast.datum.length; i++) {
        var current = parseFloat(data[id].forecast.datum[i].primary);
        if (current > crest_height) {
          crest_height = current;
          crest_i = i;
        }
      }
      if (parseFloat(crest_height) > parseFloat(data[id].current_level)) {
        data[id].has_crest    = true;
        data[id].crest_height = crest_height;
        data[id].crest_time   = data[id].forecast.datum[crest_i].valid;
        data[id].crest_state  = get_river_state(crest_height, data[id].sigstages);
      }
      else {
        data[id].has_crest = false;
      }
    }
  }

  function add_recency(data) {
    for (var id in data) {
      var update  = new Date(data[id]['@attributes'].generationtime);
      var current = new Date();
      var diff    = current.getTime() - update.getTime();

      data[id].is_recent = false;
      data[id].is_old    = false;

      if (diff < (30 * MINUTE_TICKS)) {
        data[id].is_recent = true;
      } else if (diff > (90 * MINUTE_TICKS)) {
        data[id].is_old = true;
      }
    }
  }

  $scope.get_panel_class = function(id) {
    var my_class = "panel-default";

    switch($scope.gauges[id].current_state) {
      case RIVER_STATES.ACTION:
        my_class = "panel-info";
        break;
      case RIVER_STATES.FLOOD:
        my_class = "panel-warning";
        break;
      case RIVER_STATES.MAJOR:
        my_class = "panel-danger";
        break;
    }
    return my_class;
  };
  
  $scope.get_stage_label_text = function(id) {
    var my_text = "NORMAL";

    switch($scope.gauges[id].current_state) {
      case RIVER_STATES.ACTION:
        my_text = "ACTION";
        break;
      case RIVER_STATES.FLOOD:
        my_text = "FLOOD";
        break;
      case RIVER_STATES.MAJOR:
        my_text = "MAJOR FLOOD";
        break;
    }
    return my_text;
  };

  $scope.get_stage_label_color = function(id) {
    var my_class = "";

    switch($scope.gauges[id].current_state) {
      case RIVER_STATES.ACTION:
        my_class = "label-info";
        break;
      case RIVER_STATES.FLOOD:
        my_class = "label-warning";
        break;
      case RIVER_STATES.MAJOR:
        my_class = "label-danger";
        break;
    }
    return my_class;
  };

  $scope.get_crest_label_text = function(id) {
    var my_text = "NORMAL";

    switch($scope.gauges[id].crest_state) {
      case RIVER_STATES.ACTION:
        my_text = "ACTION";
        break;
      case RIVER_STATES.FLOOD:
        my_text = "FLOOD";
        break;
      case RIVER_STATES.MAJOR:
        my_text = "MAJOR FLOOD";
        break;
    }
    return my_text;
  };

  $scope.get_crest_label_color = function(id) {
    var my_class = "";

    console.log(id);
    console.log($scope.gauges[id].crest_state);
    switch($scope.gauges[id].crest_state) {
      case RIVER_STATES.ACTION:
        my_class = "label-info";
        break;
      case RIVER_STATES.FLOOD:
        my_class = "label-warning";
        break;
      case RIVER_STATES.MAJOR:
        my_class = "label-danger";
        break;
    }
    return my_class;
  };

  $scope.get_crest_icon_color = function(id) {
    var my_class = "text-normal";

    if ($scope.gauges[id].has_crest) {
      switch($scope.gauges[id].crest_state) {
        case RIVER_STATES.ACTION:
          my_class = "text-info";
          break;
        case RIVER_STATES.FLOOD:
          my_class = "text-warning";
          break;
        case RIVER_STATES.MAJOR:
          my_class = "text-danger";
          break;
      }
    }
    return my_class;
  };

  get_endpoint_data();
  $interval(get_endpoint_data, 1 * MINUTE_TICKS);

  /* Get the data from json */
  function get_endpoint_data() {
    gaugeFactory.get()
    .then(function(gauges) {
      add_current_river_info(gauges);
      add_crest_info(gauges);
      add_recency(gauges);

      $scope.gauges = gauges;
      process_json(gauges);
    });  
  }

  function process_json(data) {
    for (var id in data) {
      var my_data = [];
      var chart_data = get_chart_arrays(data[id]);

      my_data.push(chart_data.river_level);
      my_data.push(chart_data.action);

      my_data.push(chart_data.flood);
      my_data.push(chart_data.major);

      $scope.charts[id] = {};
      $scope.charts[id].data = my_data;
      $scope.charts[id].labels = chart_data.times;
    }
  }

  function get_chart_arrays(gauge) {
    var chart_data = {river_level: [], action: [], flood: [], major: [], times: []};
    var obs_length = gauge.observed.datum.length;
    if (obs_length > gauge.forecast.datum.length) {
      obs_length = gauge.forecast.datum.length;
    }
    
    for (var i = obs_length - 1; i >= 0; i--) {
      chart_data.river_level.push(gauge.observed.datum[i].primary);
      chart_data.action.push(gauge.sigstages.action);
      chart_data.flood.push(gauge.sigstages.flood);
      chart_data.major.push(gauge.sigstages.major);
      chart_data.times.push($filter('date')(gauge.observed.datum[i].valid, "EEE HH:mm (d)"));
    }
    for (var i = 0; i < gauge.forecast.datum.length; i++) {
      chart_data.river_level.push(gauge.forecast.datum[i].primary);
      chart_data.action.push(gauge.sigstages.action);
      chart_data.flood.push(gauge.sigstages.flood);
      chart_data.major.push(gauge.sigstages.major);
      chart_data.times.push($filter('date')(gauge.forecast.datum[i].valid, "EEE HH:mm (d)"));
    }
    return chart_data;
  }

  $scope.getLocalTime = function(uctString) {
    return new Date(uctString);
  };

  $scope.getLocalDayofWeek = function(uctString) {
    var my_date = new Date(uctString);

  };

  $scope.getLocalMonthDay = function(uctString) {
    var my_date = new Date(uctString);
    return my_date.toString();
  };
})

.config(function($stateProvider) {
  $stateProvider
    /* top navigation bar */
    .state('app.precip', {
      url: "/precip",
      views: {
        'content': {
          templateUrl: "src/precip/precip.html",
          controller: "precipController"
        }
      }
    });
})

.constant('QPF', [
  {
    title: 'Day 1-2',
    url:   'data_cache/kml/Day 1-2.kml'
  },
  {
    title: 'Day 1-3',
    url:   'data_cache/kml/Day 1-3.kml'
  },
  {
    title: 'Day 1-7',
    url:   'data_cache/kml/Day 1-7.kml'
  }
  // Leaving off additional views in order to fit controls on screen
  /*,
  {
    title: 'Day 1-5',
    url:   'data_cache/kml/Day 1-5.kml'
  },
  {
    title: 'Day 4-5',
    url:   'data_cache/kml/Day 4-5.kml'
  },
  {
    title: 'Day 6-7',
    url:   'data_cache/kml/Day 6-7.kml'
  }*/
])

.controller('precipController', function($scope, $state, $window, leafletData, QPF, $timeout) {
  // This object will hold the map
  var map = {};
  // This object will hold the layers
  var qpf_layers = [];
  // Bring the QPF into scope
  $scope.QPF = QPF;
  // Initialize the forecast layer we will initially display
  $scope.my_forecast = {
    current: 0
  };
  // Initialize the height of the map view
  $scope.my_style = { 'height': '250px'};

  // Initialize the qpf_layers object
  create_layers();

  // Load qpf_layers[0] onto the map fit the layer inside the map
  leafletData.getMap('my_map').then(function(leafletMap) {
    map = leafletMap;
    qpf_layers[0].on("loaded", function(e) {
      map.fitBounds(e.target.getBounds());
      map.addLayer(qpf_layers[0]);
    });
  });
  
  // This routine creates the layer objects that are stored in qpf_layers
  function create_layers() {
    for (var i = 0; i < QPF.length; i++) {
      var my_qpf = new L.KML(QPF[i].url, {async: true});
      qpf_layers.push(my_qpf);
    }
  }

  // This routine resizes the map according to the window height
  function resize_map(windowHeight) {
    var height = Math.floor(windowHeight - 180).toString() + 'px';
    $scope.my_style.height = height;
  }

  // Watch for changes to the window height and resize the map
  $scope.$watch(function() {
    return $window.innerHeight;
  }, function(newVal, oldVal) {
    resize_map(newVal);
  });
  
  // Remove the old forecast layer by iterating through the array
  function remove_forecasts() {
    for (var i = 0; i < qpf_layers.length; i++) {
      map.removeLayer(qpf_layers[i]);
    }
  }

  // Add the forecast layer by the array index
  function add_forecast(index) {
    map.addLayer(qpf_layers[index]);
  }

  // This flag is used in the moveend event function to flag the need to invalidateSize
  var resize_flag = false;

  $scope.$on('leafletDirectiveMap.my_map.resize', function() {
    resize_flag = true;
  });

  $scope.$on('leafletDirectiveMap.my_map.moveend', function() {
    if (resize_flag === true) {
      map.invalidateSize();
      resize_flag = false;
    }
  });

  // Called by button group's ng-change function, here we change the forecast that's shown
  $scope.forecast_update = function() {
    remove_forecasts();
    add_forecast($scope.my_forecast.current);
  };

  angular.extend($scope, {
        quincy: {
            lat: 39.9356,
            lng: -91.4099,
            zoom: 3
        },
        defaults: {
            scrollWheelZoom: true
        }
    });
})

.config(function($stateProvider) {
  $stateProvider
    /* top navigation bar */
    .state('app', {
      abstract: true,
      url: "/app",
      templateUrl: "src/scaffold/scaffold.html",
      controller: "scaffoldController"
    });
})

.controller('scaffoldController', function($scope) {
  $scope.isCollapsed = false;
})

.directive('navbarDirective', function() {
  return {
    restrict: 'A',
    scope: {
      menu: '=menu'
    },
    transclude: true,
    templateUrl: "src/directives/navbar/navbar_directive.html",
    link: function(scope, elem, attrs) {
      if(attrs.ngClick || attrs.href === '' || attrs.href === '#') {
        elem.on('click', function(e){
          e.preventDefault();
        });
      }
    },
    controller: function($scope, NAVIGATION, gettextCatalog) {
      // Get the navigation tree from the NAVIGATION constant
      $scope.navigation = NAVIGATION;

      // Collapse the navbar when a link is clicked
      $scope.updateNav = function(category) {
        $scope.isCollapsed  = true;
      }

      $scope.getmyclass = function() {
        if ($scope.open) {
          return 'open';
        }
        return '';
      };
      $scope.toggleopen = function() {
        console.log($scope.open);
        if ($scope.open) {
          $scope.open = false;
        } else {
          $scope.open = true;
        }
      };
      // Get additional class(es) for <li>
      $scope.getCategoryClass = function(category) {
        var class_string = "";
        if (category.hasOwnProperty('children')) {
          class_string = "dropdown";
        }
        return class_string;
      };
    }
  };
})

.constant('NAVIGATION', [
  {
    description: gettext('Gauges'),
    link:        'app.home'
  },
  {
    description: gettext('Precipitation Forecast'),
    link:        'app.precip'
  }/*,
  {
    description: gettext('Example'),
    children: [
      {
        description: gettext('Submenu Link'),
        link:        'app.example'
      },
    ]
  }*/
])
